# -*- coding: utf-8 -*-

from django_translate.services import translator as django_translator

class LocaleMiddleware(object):
    """
    This is a very simple middleware that parses a request
    and decides what locale to activate in current django_translator
    """
    def process_request(self, request):
        if not hasattr(request, 'LANGUAGE_CODE'):
            raise RuntimeError('django.middleware.locale.LocaleMiddleware is required '
                               'in order to use django_translate.middleware.LocaleMiddleware ')
        django_translator.locale = request.LANGUAGE_CODE



# -*- coding: utf-8 -*-
from __future__ import absolute_import

import re
import json
from functools import partial
from django import template
from django.core.urlresolvers import reverse as _reverse
from django.template import Library, Node, TemplateSyntaxError
from django.template.loader import render_to_string
from django_translate.services import translator
from django_translate import settings

register = Library()


def tranz(parser, token, is_transchoice=False):
    """
    Templatetagish wrapper for Translator.trans()

    :param parser:
    :param token:
    :param is_transchoice:
    :return:
    """
    tokens = token.split_contents()
    id = tokens[1]
    number = domain = locale = None
    parameters = {}
    if len(tokens) > 2:
        skip_idx = None
        for idx, token in enumerate(tokens[2:], start=2):
            if idx == skip_idx:
                skip_idx = None
                continue

            if "=" in token:
                k, v = token[0:token.index('=')], token[token.index('=') + 1:]
                parameters[k] = v
            elif token == "number":
                number = tokens[idx + 1]
                skip_idx = idx + 1
            elif token == "from":
                domain = tokens[idx + 1]
                skip_idx = idx + 1
            elif token == "into":
                locale = tokens[idx + 1]
                skip_idx = idx + 1
            else:
                raise TemplateSyntaxError(
                    "Unexpected token {0} in tag tranz".format(token))
    if isinstance(number, str):
        import ipdb
        ipdb.set_trace()
    if is_transchoice and number is None:
        raise TemplateSyntaxError(
            "number parameter expected in tag {tag_name}")

    return TranzNode(
        id,
        parameters,
        domain,
        locale,
        number,
        is_transchoice=is_transchoice)

register.tag("tranz", tranz)
register.tag("tranzchoice", partial(tranz, is_transchoice=True))


class TranzNode(Node):

    def __init__(
            self,
            id,
            parameters,
            domain,
            locale,
            number=None,
            is_transchoice=False):
        self.id = template.Variable(id)
        self.parameters = {
            k: template.Variable(v) for k,
            v in parameters.items()}
        self.domain = domain
        self.locale = locale
        self.is_transchoice = is_transchoice
        self.number = number if is_transchoice else None
        super(TranzNode, self).__init__()

    def render(self, context):
        prefix = context.get("tranz_prefix", "")
        if prefix:
            prefix += "_"
        id = prefix + self.id.resolve(context)
        parameters = {
            k: v.resolve(context) for k,
            v in self.parameters.items()}
        domain = template.Variable(
            self.domain).resolve(context) if self.domain is not None else context.get(
            'tranz_domain',
            None)
        locale = template.Variable(
            self.locale).resolve(context) if self.locale is not None else context.get(
            'tranz_locale',
            None)
        number = template.Variable(self.number).resolve(
            context) if self.number is not None else None

        if locale is None:
            # Try to use LocaleMiddleware if it's on
            if isinstance(
                    context,
                    template.RequestContext) and hasattr(
                    context.request,
                    'LANGUAGE_CODE'):
                locale = context.request.LANGUAGE_CODE
        if locale is None:
            locale = settings.TRANZ_DEFAULT_LANGUAGE

        if self.is_transchoice:
            return translator.transchoice(
                id,
                number,
                parameters,
                domain,
                locale)
        else:
            return translator.trans(id, parameters, domain, locale)


@register.tag
def tranz_context(parser, token):
    """
    Templatetagish wrapper for Translator.transchoice()
    """
    tokens = token.split_contents()

    parameters = {}
    for idx, token in enumerate(tokens[1:], start=1):
        if "=" in token:
            if token[0:token.index('=')] not in ("domain", "prefix", "locale"):
                raise TemplateSyntaxError(
                    "Unexpected token {0} in tag {{tag_name}}".format(token))

            k, v = token[0:token.index('=')], token[token.index('=') + 1:]
            parameters[k] = v
        else:
            raise TemplateSyntaxError(
                "Unexpected token {0} in tag {{tag_name}}".format(token))

    return TranzContextNode(
        parameters.get(
            'prefix', None), parameters.get(
            'domain', None), parameters.get(
                'locale', None))


class TranzContextNode(Node):

    def __init__(self, prefix, domain, locale):
        self.prefix = prefix
        self.domain = domain
        self.locale = locale
        super(TranzContextNode, self).__init__()

    def render(self, context):
        if self.prefix is not None:
            context['tranz_prefix'] = template.Variable(
                self.prefix).resolve(context)

        if self.domain is not None:
            context['tranz_domain'] = template.Variable(
                self.domain).resolve(context)

        if self.locale is not None:
            context['tranz_locale'] = template.Variable(
                self.locale).resolve(context)
        return ""



# -*- coding: utf-8 -*-
"""
Do not import before django is initialized or you will run into
circular imports
"""

import os
from collections import defaultdict

from django.apps.registry import apps
from python_translate import loaders
from python_translate import translations

from django_translate import settings


def discover_resources():
    """
    Searches for translations files matching [catalog].[lang].[format]

    Traverses TRANZ_LOCALE_PATHS:
    -
    | TRANZ_LOCALE_PATHS
     +- messages.fr.yml
      | messages.en.yml

    And apps paths if TRANZ_SEARCH_LOCALE_IN_APPS is set to True):
    -
    | app_path
    +- TRANZ_DIR_NAME
     +- messages.fr.yml
      | messages.en.yml

    @rtype: list
    @return: A list of all found translation files
    """
    locale_discovery_paths = settings.TRANZ_LOCALE_PATHS
    if settings.TRANZ_SEARCH_LOCALE_IN_APPS:
        locale_discovery_paths += [os.path.join(app.path, '/', settings.TRANZ_DIR_NAME ) for app in apps.app_configs.values()]

    resources = []
    for path in locale_discovery_paths:
        if not os.path.isdir(path):
            continue

        ls = os.listdir(path)

        # Try to match direct children or discovery paths
        for file in ls:
            if os.path.isfile(os.path.join(path, file)):
                try:
                    domain, lang, format = file.split('.')
                except ValueError as e:
                    continue
                resources.append((format, os.path.join(path, file), lang, domain))
    return resources


translator = settings.TRANZ_TRANSLATOR_CLASS(settings.TRANZ_DEFAULT_LANGUAGE)
for format, loader in settings.TRANZ_LOADERS.items():
    translator.add_loader(format, loader)

for format, path, locale, domain in discover_resources():
    translator.add_resource(format, path, locale, domain)

_ = translator.trans
trans = translator.trans
transchoice = translator.transchoice

loader = settings.TRANZ_LOADER_CLASS()
for format, subloader in settings.TRANZ_LOADERS.items():
    loader.add_loader(format, subloader)

writer = settings.TRANZ_WRITER_CLASS()
for format, subdumper in settings.TRANZ_DUMPERS.items():
    writer.add_dumper(format, subdumper)

extractor = settings.TRANZ_EXTRACTOR_CLASS()
for format, subextractor in settings.TRANZ_EXTRACTORS.items():
    extractor.add_extractor(format, subextractor)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
import os.path
import re
import sys
import string

from django.apps.registry import apps
from django.core.management.base import BaseCommand, CommandError
from python_translate.extractors import base as extractors
from python_translate import operations
from python_translate.translations import MessageCatalogue

import django_translate
from django_translate.utils import bcolors
from django_translate import services
from django_translate import settings


class AnyFormatSpec:

    def __format__(self, fmt):
        return ''


class Formatter(string.Formatter):

    def __init__(self):
        self.used = set()

    def get_value(self, key, args, kwargs):
        self.used.add(key)
        return AnyFormatSpec()


class Command(BaseCommand):

    help = """Validates translation strings from templates from a given location.

              Example running against app folder
                  ./manage.py tranzvalidate -l en --app website
                  ./manage.py tranzvalidate -l fr --path ./ --tranz-dir ./tranz --exclude ./static
              """

    def __init__(self, stdout=None, stderr=None, no_color=False):
        self.excluded_paths = None
        self.locale = None
        self.verbosity = None

        super(Command, self).__init__(stdout, stderr, no_color)


    def add_arguments(self, parser):
        parser.add_argument('--locale', '-l', default='en', dest='locale', action='store',
                            help='Locale to process')

        parser.add_argument('--app', '-a', dest='app', action='store',
                            help='App to scan.')

        parser.add_argument('--path', '-p', dest='path', action='store',
                            help='Path to scan')

        parser.add_argument('--tranz-dir', dest='tranz_dir', default=None, action='store',
                            help='Override the default tranz dir (current translations will be loaded from it)')

        parser.add_argument('--exclude-dir', '-x', default=[], dest='excluded_paths', action='append',
                            help='Paths to exclude. Default is none. Can be used multiple times. '
                                 'Works only with ChainExtractor.')


    def handle(self, *args, **options):
        if not (bool(options.get('app')) ^ bool(options.get('path'))):
            print bcolors.WARNING + 'You must choose only one of --app or --path' + bcolors.ENDC
            return

        if not options.get('tranz_dir') and (not options.get('app') or not settings.TRANZ_SEARCH_LOCALE_IN_APPS):
            print bcolors.WARNING + 'You must provide an --tranz-dir when in --path mode, or when TRANZ_SEARCH_LOCALE_IN_APPS ' \
                                    'settings variable is False.' + bcolors.ENDC
            return

        self.excluded_paths = [os.path.abspath(path) for path in options['excluded_paths']]
        self.excluded_paths += [os.path.abspath(django_translate.__path__[0])]
        self.locale = options['locale']
        self.verbosity = options['verbosity']

        # Find directories to scan
        if options.get('app'):
            for app in apps.app_configs.values():
                if app.name == options.get('app'):
                    current_name = app.name
                    root_path = app.path
                    break
            else:
                raise ValueError("App {0} not found".format(options.get('app')))
        else:
            root_path = os.path.abspath(options['path'])
            current_name = root_path.split("/")[-1]

        tranz_dir = options.get('tranz_dir') or os.path.join(root_path, 'tranz')

        print "Loading existing messages"
        current_catalogue = MessageCatalogue(options['locale'])
        loader = services.loader
        loader.load_messages(tranz_dir, current_catalogue)
        if len(current_catalogue.messages) == 0:
            print "No messages were loaded, make sure there actually are " \
                  "translation file in format {{catalog}}.{{locale}}.{{format}} in {0}".format(tranz_dir)
            return

        print "Extracting translations"
        translations = self.extract_translations(services.extractor, root_path)
        if len(translations) == 0:
            print "No messages were extracted, from {0} using {1}".format(root_path, services.extractor.__class__.__name__)
            return

        self.validate_translations(translations, current_catalogue)

    def extract_translations(self, extractor, root_path):
        if isinstance(extractor, extractors.ChainExtractor):
            subextractors = extractor._extractors.values()
        else:
            subextractors = [extractor]

        translations = []

        for subextractor in subextractors:
            if not isinstance(subextractor, extractors.BaseExtractor):
                print "Skipping extractor ", subextractor.__type__.__name__
                continue

            paths = subextractor.extract_files(root_path)
            paths = self.filter_exluded_paths(paths)
            for path in paths:
                try:
                    with open(path, 'r') as f:
                        batch = subextractor.extract_translations(f.read())
                        for t in batch:
                            t.file = path
                        translations += batch
                except Exception as e:
                    exc_type, exc_value, exc_traceback = sys.exc_info()
                    msg = 'There was an exception in extractor {0} when processing ' \
                          'resource "{1}"'.format(type(subextractor).__name__, path)
                    msg = msg + "\nOriginal message: {0} {1}".format(exc_type.__name__, exc_value)
                    raise ValueError(msg), None, exc_traceback
        return translations

    def validate_translations(self, translations, current_catalogue):
        """
        @TODO: check if translation is present at all languages
        @TODO: check if all plural forms are present
        :param translations:
        @return:
        """
        self.flushed = 0

        file = None
        warnings = []
        for t in translations:
            if t.file != file:
                self.flush_warnings(file, warnings)
                warnings = []
                file = t.file

            if t.id.is_literal:
                domain = t.domain.value if t.domain and t.domain.is_literal else "messages"

                if not current_catalogue.has(t.id.value, domain):
                    warnings.append(['No such translation "{0}" in domain {1}'.format(t.id.value, domain), t])
                else:
                    formatter = Formatter()
                    formatter.format(current_catalogue.get(t.id.value, domain))

                    if len(formatter.used) and not t.parameters:
                        warnings.append(["No format variables passed, expected: {0}".format(", ".join(formatter.used)), t])

                    if len(formatter.used) and t.parameters and t.parameters.is_literal:
                        parameters = set(t.parameters.value)
                        if t.is_transchoice and t.number is not None:
                            parameters.add("count")

                        if formatter.used != parameters:
                            warnings.append(["Expected/received parameters mismatch, (expected: {0}), (received: {1})".format(
                                ", ".join(formatter.used), ", ".join(parameters)
                            ), t])

                    if t.is_transchoice and t.number is None:
                        warnings.append(['Transchoice "number" is missing', t])

            if t.id and not t.id.is_literal:
                warnings.append(["Translation of non-literal value", t])

            if t.parameters and not t.parameters.is_literal:
                warnings.append(["Variables passed via reference", t])
        self.flush_warnings(file, warnings)

        if self.flushed == 0:
            print bcolors.OKGREEN + "Scanned {0} translations and everything seems to be ok!".format(len(translations)) + bcolors.ENDC
        else:
            print bcolors.FAIL + "{0} problems found".format(self.flushed) + bcolors.ENDC

    def filter_exluded_paths(self, paths):
        valid = []
        for path in paths:
            for excluded in self.excluded_paths:
                if path.startswith(excluded):
                    break
            else:
                valid.append(path)
        return valid

    def flush_warnings(self, file, warnings):
        if not len(warnings):
            return

        self.flushed += len(warnings)

        lines = None
        if self.verbosity > 1:
            try:
                with open(file, 'r') as f:
                    lines = f.read().split("\n")
            except Exception, e:
                pass

        print bcolors.WARNING + "{0} warnings in file {1}".format(len(warnings), file) + bcolors.ENDC
        for msg, trans in warnings:
            print bcolors.WARNING + "line {0}: {1}".format(trans.lineno or "?", msg) + bcolors.ENDC
            if lines and self.verbosity > 1 and trans.lineno:
                print "{0}: {1}".format(trans.lineno, lines[trans.lineno-1])

        print ""



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
import os.path
import re
import sys
import string

from django.apps.registry import apps
from django.core.management.base import BaseCommand, CommandError
from python_translate.extractors import base as extractors
from python_translate import operations
from python_translate.translations import MessageCatalogue

from django_translate.utils import bcolors
from django_translate import services
from django_translate import settings


class AnyFormatSpec:

    def __format__(self, fmt):
        return ''


class Formatter(string.Formatter):

    def __init__(self):
        self.used = set()

    def get_value(self, key, args, kwargs):
        self.used.add(key)
        return AnyFormatSpec()


class Command(BaseCommand):

    help = """Extract translation strings from templates from a given location. It can display them or merge
              the new ones into the translation files. When new translation strings are found it can
              automatically add a prefix to the translation message.

              Example running against app folder
                  ./manage.py tranzdump -l en --path ./ --output-path ./tranz
                  ./manage.py tranzdump -l fr --force --prefix="new_" --app website --exclude ./website/static
              """

    def __init__(self, stdout=None, stderr=None, no_color=False):
        self.excluded_paths = None
        self.locale = None
        self.verbosity = None
        super(Command, self).__init__(stdout, stderr, no_color)


    def add_arguments(self, parser):
        parser.add_argument('--locale', '-l', default='en', dest='locale', action='store',
                            help='Locale to process')

        parser.add_argument('--app', '-a', dest='app', action='store',
                            help='App to scan.')

        parser.add_argument('--path', '-p', dest='path', action='store',
                            help='Path to scan')

        parser.add_argument('--output-dir', dest='output_dir', default=None, action='store',
                            help='Override the default output dir')

        parser.add_argument('--exclude-dir', '-x', default=[], dest='excluded_paths', action='append',
                            help='Paths to exclude. Default is none. Can be used multiple times. '
                                 'Works only with ChainExtractor.')

        parser.add_argument('--prefix', dest='prefix', default="__", action='store',
                            help='Override the default prefix')

        parser.add_argument('--format', dest='format', default="yml", action='store',
                            help='Override the default output format')

        parser.add_argument('--dump-messages', dest='dump_messages', action='store_true',
                            help='Should the messages be dumped in the console')

        parser.add_argument('--force', dest='force', action='store_true',
                            help='Should the update be done')

        parser.add_argument('--no-backup', dest='no_backup', action='store_true',
                            help='Should backup be disabled')

        parser.add_argument('--clean', dest='clean', default=False, action='store_true',
                            help='Should clean not found messages',)

    def handle(self, *args, **options):
        if options.get('force') != True and options.get('dump_messages') != True:
            print bcolors.WARNING + 'You must choose at least one of --force or --dump-messages' + bcolors.ENDC
            return

        if not (bool(options.get('app')) ^ bool(options.get('path'))):
            print bcolors.WARNING + 'You must choose only one of --app or --path' + bcolors.ENDC
            return

        if not options.get('output_dir') and (not options.get('app') or not settings.TRANZ_SEARCH_LOCALE_IN_APPS):
            print bcolors.WARNING + 'You must provide an --output-dir when in --path mode, or when TRANZ_SEARCH_LOCALE_IN_APPS ' \
                                    'settings variable is False.' + bcolors.ENDC
            return

        self.excluded_paths = [os.path.abspath(path) for path in options['excluded_paths']]

        # Find directories to scan
        if options.get('app'):
            for app in apps.app_configs.values():
                if app.name == options.get('app'):
                    current_name = app.name
                    root_path = app.path
                    break
            else:
                raise ValueError("App {0} not found".format(options.get('app')))
        else:
            root_path = os.path.abspath(options['path'])
            current_name = root_path.split("/")[-1]

        output_dir = options.get('output_dir') or os.path.join(root_path, 'tranz')
        writer = services.writer

        print 'Generating "{0}" translation files for "{1}"'.format(options.get('locale'), current_name)

        print "Loading existing messages"
        current_catalogue = MessageCatalogue(options['locale'])
        loader = services.loader
        loader.load_messages(output_dir, current_catalogue)
        if len(current_catalogue.messages) == 0:
            print "No messages were loaded, make sure there actually are " \
                  "translation file in format {{catalog}}.{{locale}}.{{format}} in {0}".format(output_dir)
            return

        print "Extracting messages"
        extracted_catalogue = MessageCatalogue(options['locale'])
        extractor = services.extractor
        extractor.set_prefix(options['prefix'])
        self.extract_messages(extractor, root_path, extracted_catalogue)

        print "Processing catalogues"
        operation_class = operations.DiffOperation if options['clean'] else operations.MergeOperation
        operation = operation_class(current_catalogue, extracted_catalogue)

        if not len(operation.get_domains()):
            print "No translations found"
            return

        if options["dump_messages"]:
            for domain in operation.get_domains():
                print "Displaying messages for domain {0}".format(domain)
                new_keys = operation.get_new_messages(domain).keys()
                all_keys = operation.get_messages(domain).keys()
                for id in set(all_keys).difference(new_keys):
                    print id

                for id in new_keys:
                    print bcolors.OKGREEN + id + bcolors.ENDC

                for id in operation.get_obsolete_messages(domain).keys():
                    print bcolors.FAIL + id + bcolors.ENDC

        if options["no_backup"]:
            writer.disable_backup()

        if options["force"]:
            print "Writing files to {0}".format(output_dir)
            writer.write_translations(operation.get_result(), options['format'], {
                "path": output_dir,
                "default_locale": options['locale']
            })

    def extract_messages(self, extractor, root_path, extracted_catalogue):
        if isinstance(extractor, extractors.ChainExtractor):
            subextractors = extractor._extractors.values()
        else:
            subextractors = [extractor]

        for subextractor in subextractors:
            if not isinstance(subextractor, extractors.BaseExtractor):
                subextractor.extract(root_path, extracted_catalogue)
                continue

            paths = subextractor.extract_files(root_path)
            paths = self.filter_exluded_paths(paths)
            for path in paths:
                try:
                    subextractor.extract([path], extracted_catalogue)
                except Exception, e:
                    exc_type, exc_value, exc_traceback = sys.exc_info()
                    msg = 'There was an exception in extractor {0} when processing ' \
                          'resource "{1}"'.format(type(subextractor).__name__, path)
                    msg = msg + "\nOriginal message: {0} {1}".format(exc_type.__name__, exc_value)
                    raise ValueError(msg), None, exc_traceback

    def filter_exluded_paths(self, paths):
        valid = []
        for path in paths:
            for excluded in self.excluded_paths:
                if path.startswith(excluded):
                    break
            else:
                valid.append(path)
        return valid



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

setup(
    name="django-translate",
    version="1.0.4",
    author="Adam Zieliński",
    author_email="adam@sf2.guru",
    packages=find_packages(),
    include_package_data=True,
    url="https://github.com/adamziel/django_translate",

    license="MIT",
    description="Non-gettext translations for django.",

    # Dependent packages (distributions)
    install_requires=[
        "python_translate"
    ],
)



# -*- coding: utf-8 -*-
import os
from django.conf import settings
import python_translate.extractors as extractors
import python_translate.extractors.base
import python_translate.extractors.python
import django_translate.extractors.django_template

import python_translate.dumpers as dumpers

from python_translate import loaders
from python_translate import translations
from python_translate import glue

def _d(param, default_value):
    if hasattr(settings, param):
        return getattr(settings, param)
    else:
        return default_value()

TRANZ_EXCLUDED_DIRS = _d('TRANZ_EXCLUDED_DIRS', lambda: [])
TRANZ_EXCLUDED_DIRS.append(os.path.dirname(os.path.abspath(__file__)))

TRANZ_LOADERS = _d('TRANZ_LOADERS', lambda: {
    "json": loaders.JSONFileLoader(),
    "yml": loaders.YamlFileLoader()
})

TRANZ_DUMPERS = _d('TRANZ_DUMPERS', lambda: {
    "json": dumpers.JSONFileDumper(),
})

TRANZ_EXTRACTORS = _d('TRANZ_EXTRACTORS', lambda: {
    "template": django_translate.extractors.django_template.DjangoTemplateExtractor(),
    "py": extractors.python.PythonExtractor()
})

TRANZ_LOADER_CLASS = _d('TRANZ_LOADER_CLASS', lambda: glue.TransationLoader)
TRANZ_WRITER_CLASS = _d('TRANZ_WRITER_CLASS', lambda: glue.TranslationWriter)

TRANZ_EXTRACTOR_CLASS = _d('TRANZ_EXTRACTOR_CLASS',
                            lambda: extractors.base.ChainExtractor)

TRANZ_TRANSLATOR_CLASS = _d('TRANZ_TRANSLATOR_CLASS',
                             lambda: translations.Translator)

TRANZ_SEARCH_LOCALE_IN_APPS = _d('TRANZ_SEARCH_LOCALE_IN_APPS', lambda: True)
TRANZ_DIR_NAME = _d('TRANZ_DIR_NAME', lambda: 'tranz')

TRANZ_LOCALE_PATHS = _d('TRANZ_LOCALE_PATHS', lambda: [])

if not isinstance(TRANZ_LOCALE_PATHS, (tuple, list)):
    raise ValueError("TRANZ_LOCALE_PATHS must be a tuple or a list")

TRANZ_DEFAULT_LANGUAGE = _d('TRANZ_DEFAULT_LANGUAGE', lambda: "en")




# -*- coding: utf-8 -*-

import re
import os
from django.template.base import Lexer, TOKEN_TEXT, TOKEN_VAR, TOKEN_BLOCK
from python_translate.extractors.base import Translation, TransVar, ExtensionBasedExtractor

val = '''(?:[^"' ]+)|(?:"[^"]*?")|(?:'[^']*?')'''

id_re = re.compile(r"^tranz(?:choice)?\s*({0})".format(val))
number_re = re.compile(r"number\s*({0})".format(val))
domain_re = re.compile(r"from\s*({0})".format(val))
locale_re = re.compile(r"into\s*({0})".format(val))
properties_re = re.compile(r"(?:\s+\w+=(?:{0}))".format(val))


class DjangoTemplateExtractor(ExtensionBasedExtractor):

    def __init__(
            self,
            file_extensions=None,
            tranz_tag=None,
            tranzchoice_tag=None):
        file_extensions = file_extensions if file_extensions is not None else (
            "*.html",
            "*.txt")

        self.tranz_tag = tranz_tag if tranz_tag is not None else 'tranz'
        self.tranzchoice_tag = tranzchoice_tag if tranzchoice_tag is not None else 'tranzchoice'
        super(DjangoTemplateExtractor, self).__init__(file_extensions=file_extensions)

    def extract_translations(self, string):
        """Extract messages from Django template string."""

        trans = []
        for t in Lexer(string, None).tokenize():
            if t.token_type == TOKEN_BLOCK:
                if not t.contents.startswith(
                        (self.tranz_tag, self.tranzchoice_tag)):
                    continue

                is_tranzchoice = t.contents.startswith(
                    self.tranzchoice_tag +
                    " ")
                kwargs = {
                    "id": self._match_to_transvar(id_re, t.contents),
                    "number": self._match_to_transvar(number_re, t.contents),
                    "domain": self._match_to_transvar(domain_re, t.contents),
                    "locale": self._match_to_transvar(locale_re, t.contents),
                    "is_transchoice": is_tranzchoice, "parameters": TransVar(
                        [x.split("=")[0].strip() for x in properties_re.findall(t.contents) if x],
                        TransVar.LITERAL
                    ),
                    "lineno": t.lineno,
                }

                trans.append(Translation(**kwargs))
        return trans

    def _match_to_transvar(self, reg, string, default=""):
        match = reg.findall(string)
        return self._to_transvar(match[0] if match else default)

    def _to_transvar(self, val):
        if val is None:
            return None

        if len(val) > 0 and (val[0] in ('"', "'") or val.isdigit()):
            if val[0] in ('"', "'"):
                val = val[1:-1]
            return TransVar(val, TransVar.LITERAL)
        else:
            return TransVar(val, TransVar.VARNAME)
